@@ -94,10 +94,18 @@ ipmi_log(enum ipmi_log_type_e log_type, const char *format, ...)
     va_list ap;
 
     va_start(ap, format);
-    if (ipmi_os_handler->vlog)
+    if (ipmi_os_handler && ipmi_os_handler->vlog)
 	ipmi_os_handler->vlog(ipmi_os_handler, log_type, format, ap);
-    else
+    else {
 	vfprintf(stderr, format, ap);
+	switch (log_type) {
+	case IPMI_LOG_DEBUG_START:
+	case IPMI_LOG_DEBUG_CONT:
+	    break;
+	default:
+	    fprintf(stderr, "\n");
+	}
+    }
     va_end(ap);
 }
 
@@ -919,6 +927,10 @@ void _ipmi_normal_fru_shutdown(void);
 void _ipmi_fru_spd_decoder_shutdown(void);
 int _ipmi_sol_init(void);
 
+void _ipmi_rakp_shutdown(void);
+void _ipmi_aes_cbc_shutdown(void);
+void _ipmi_hmac_shutdown(void);
+void _ipmi_md5_shutdown(void);
 void ipmi_oem_atca_conn_shutdown(void);
 void ipmi_oem_intel_shutdown(void);
 void ipmi_oem_kontron_conn_shutdown(void);
@@ -953,6 +965,8 @@ ipmi_init(os_handler_t *handler)
     if (rv)
 	return rv;
 
+    ipmi_initialized = 1;
+
     if (handler->create_lock) {
 	rv = handler->create_lock(handler, &seq_lock);
 	if (rv)
@@ -1012,28 +1026,10 @@ ipmi_init(os_handler_t *handler)
     ipmi_oem_atca_init();
     init_oem_test();
 
-    ipmi_initialized = 1;
-
     return 0;
 
  out_err:
-    _ipmi_sol_shutdown();
-#ifdef HAVE_OPENIPMI_SMI
-    _ipmi_smi_shutdown();
-#endif
-    _ipmi_lan_shutdown();
-    ipmi_oem_intel_shutdown();
-    ipmi_oem_kontron_conn_shutdown();
-    _ipmi_mc_shutdown();
-    _ipmi_domain_shutdown();
-    _ipmi_fru_spd_decoder_shutdown();
-    _ipmi_normal_fru_shutdown();
-    _ipmi_fru_shutdown();
-    if (seq_lock)
-	handler->destroy_lock(ipmi_os_handler, seq_lock);
-    if (con_type_list)
-	locked_list_destroy(con_type_list);
-    ipmi_os_handler = NULL;
+    ipmi_shutdown();
     return rv;
 }
 
@@ -1043,6 +1039,10 @@ ipmi_shutdown(void)
     if (! ipmi_initialized)
 	return;
 
+    _ipmi_rakp_shutdown();
+    _ipmi_aes_cbc_shutdown();
+    _ipmi_hmac_shutdown();
+    _ipmi_md5_shutdown();
     _ipmi_sol_shutdown();
     _ipmi_lan_shutdown();
 #ifdef HAVE_OPENIPMI_SMI
@@ -1054,6 +1054,7 @@ ipmi_shutdown(void)
     ipmi_oem_kontron_conn_shutdown();
     _ipmi_mc_shutdown();
     _ipmi_domain_shutdown();
+    _ipmi_fru_spd_decoder_shutdown();
     _ipmi_conn_shutdown();
     _ipmi_normal_fru_shutdown();
     _ipmi_fru_shutdown();
@@ -1062,6 +1063,8 @@ ipmi_shutdown(void)
     if (con_type_list)
 	locked_list_destroy(con_type_list);
 
+    ipmi_os_handler = NULL;
+
     ipmi_initialized = 0;
 }
 
@@ -1517,6 +1520,12 @@ ipmi_parse_options(ipmi_open_option_t *option,
     } else if (strcmp(arg, "-localonly") == 0) {
 	option->option = IPMI_OPEN_OPTION_LOCAL_ONLY;
 	option->ival = 1;
+    } else if (strcmp(arg, "-nocache") == 0) {
+	option->option = IPMI_OPEN_OPTION_USE_CACHE;
+	option->ival = 0;
+    } else if (strcmp(arg, "-cache") == 0) {
+	option->option = IPMI_OPEN_OPTION_USE_CACHE;
+	option->ival = 1;
     } else
 	return EINVAL;
 
@@ -1534,8 +1543,10 @@ ipmi_parse_options_help(void)
 	"-[no]ipmbscan - IPMB bus scanning\n"
 	"-[no]oeminit - special OEM processing (like ATCA)\n"
 	"-[no]seteventrcvr - setting event receivers\n"
+	"-[no]setseltime - setting the SEL clock\n"
 	"-[no]activate - connection activation\n"
 	"-[no]localonly - Just talk to the local BMC, (ATCA-only, for blades)\n"
+        "-[no]cache - use the local cache for SDRs.  On by default."
 	"-wait_til_up - wait until the domain is up before returning";
 }
 
